package com.lantone.qc.kernel.structure.ai.process;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.lantone.qc.kernel.structure.ai.model.CrfOut;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description:
 * @Author: HUJING
 * @Date: 2020/2/2 10:07
 */
public abstract class EntityProcess {
    public abstract void extractEntity(JSONObject outputs, CrfOut outputInfo);

    /**
     * 处理关系抽取输出的json
     *
     * @param outputs    关系抽取输出的json
     * @param entityType 需要处理的实体类别
     * @return
     */
    public List<Map<String, String>> processJson(JSONObject outputs, String entityType) {
        List<Map<String, String>> connectEntityList = new ArrayList<>();
        Map<String, String> connectEntity = null;
        JSONObject annotation = outputs.getJSONObject("annotation");
        JSONArray entitys = annotation.getJSONArray("T");
        JSONArray relations = annotation.getJSONArray("R");
        for (int i = 0; i < entitys.size(); i++) {
            if (StringUtils.isEmpty(entitys.get(i).toString())) {
                continue;
            }
            JSONObject entity = entitys.getJSONObject(i);
            if (entityType.equals(entity.getString("name"))) {
                int id = entity.getIntValue("id");
                List<Integer> connectEntityIdList = getConnectEntityIdList(id, relations);
                if (connectEntityIdList.size() == 0) {
                    connectEntity = new HashMap<>();
                    connectEntity.put(entity.getString("name"), entity.getString("value"));
                    connectEntityList.add(connectEntity);
                } else {
                    connectEntity = getConnectEntity(connectEntityIdList, entitys);
                    connectEntity.put(entity.getString("name"), entity.getString("value"));
                    connectEntityList.add(connectEntity);
                }
            }
        }
        return connectEntityList;
    }

    /**
     * 获取与传入实体有关系实体的id列表(List)
     *
     * @param entityId  传入实体的id
     * @param relations 关系抽取出的关系对
     * @return connectEntityIdList 有关系实体的id列表(List)
     */
    public List<Integer> getConnectEntityIdList(int entityId, JSONArray relations) {
        List<Integer> connectEntityIdList = new ArrayList<>();
        for (int i = 0; i < relations.size(); i++) {
            if (StringUtils.isEmpty(relations.get(i).toString())) {
                continue;
            }
            JSONObject relation = relations.getJSONObject(i);
            if (relation.getIntValue("from") == entityId) {
                connectEntityIdList.add(relation.getIntValue("to"));
            }
            if (relation.getIntValue("to") == entityId) {
                connectEntityIdList.add(relation.getIntValue("from"));
            }
        }
        return connectEntityIdList;
    }

    /**
     * 获取实体id列表对应的所有实体类型及实体值
     *
     * @param connectEntityIdList 实体id列表
     * @param entitys             关系抽取的实体列表
     * @return entityRelationPair 实体id列表对应的所有实体类型及实体值
     */
    public Map<String, String> getConnectEntity(List<Integer> connectEntityIdList, JSONArray entitys) {
        Map<String, String> entityRelationPair = new HashMap<>();
        for (int connectEntityId : connectEntityIdList) {
            for (int i = 0; i < entitys.size(); i++) {
                if (StringUtils.isEmpty(entitys.get(i).toString())) {
                    continue;
                }
                JSONObject entity = entitys.getJSONObject(i);
                if (connectEntityId == entity.getIntValue("id")) {
                    if (entityRelationPair.containsKey(entity.getString("name"))) {
                        entityRelationPair.put(entity.getString("name"),
                                entityRelationPair.get(entity.getString("name")) + "," + entity.getString("value"));
                    } else {
                        entityRelationPair.put(entity.getString("name"), entity.getString("value"));
                    }
                    break;
                }
            }
        }
        return entityRelationPair;
    }


    public String[] extract_digit(String value) {
        String[] res = new String[2];
        try {
            String reg_time = "([\\d]+)([\\u4e00-\\u9fa5]+)";
            Pattern pattern = Pattern.compile(reg_time);
            Matcher matcher = pattern.matcher(value);
            if (matcher.find(0)) {
                res[0] = matcher.group(1);
                res[1] = matcher.group(2);
            } else {
                res[0] = value;
                res[1] = "";
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return res;
    }
}
